"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var t = _interopRequireWildcard(require("@babel/types"));

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _util = require("./util");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 *
 */
function getEvacuatedCommentText(type, identifier, mod = '', {
  evacuatedTagName
}) {
  return `
 * @preserve
 * @${evacuatedTagName} ${type} ${identifier}${mod}
 `;
}
/**
 *
 */


function createEvacuatedCallNode(args, {
  evacuatedTagName
}) {
  return t.callExpression(t.memberExpression(t.thisExpression(), t.identifier(`__${evacuatedTagName}__`)), args);
}
/**
 *
 */


function createEvacuatedClassMethodNode(args, {
  evacuatedTagName,
  index
}) {
  return t.classMethod('method', t.identifier(`__${evacuatedTagName}_${index}`), [], t.blockStatement([t.expressionStatement(createEvacuatedCallNode(args, {
    evacuatedTagName
  }))]));
}
/**
 *
 */


function createDecoratorNode(node) {
  const {
    expression
  } = node;
  const elements = [];

  if (expression.type === 'Identifier') {
    const identifier = expression.name;
    elements.push(t.stringLiteral(identifier));
  } else if (expression.type === 'CallExpression') {
    const identifier = expression.callee.name;
    elements.push(t.stringLiteral(identifier));
    elements.push(...expression.arguments.map(a => (0, _util.stripLocationFromNode)(a)));
  }

  return t.arrayExpression(elements);
}
/**
 *
 */


function evacuateClassPropNode(node, options) {
  const {
    decorators = [],
    key: {
      name: propName
    }
  } = node;
  const args = [t.stringLiteral('classProperty'), t.stringLiteral(propName), t.arrayExpression(decorators.map(createDecoratorNode))];

  if (node.value) {
    args.push((0, _util.stripLocationFromNode)(node.value));
  }

  return createEvacuatedClassMethodNode(args, options);
}
/**
 *
 */


function evacuateClassMethodDecoratorsNode(node, options) {
  const {
    decorators = [],
    key: {
      name: methodName
    }
  } = node;
  const args = [t.stringLiteral('classMethod'), t.stringLiteral(methodName), t.arrayExpression(decorators.map(createDecoratorNode))];
  return createEvacuatedClassMethodNode(args, options);
}
/**
 *
 */


const evacuateFeaturesPlugin = (0, _helperPluginUtils.declare)((api, options = {}) => {
  api.assertVersion(7);
  const {
    features = ['classProperties', 'decorator'],
    evacuatedTagName = 'evacuated'
  } = options;
  let evacuateIndex = 0;
  return {
    name: 'transform-evacuate-features',
    visitor: {
      ClassBody() {
        evacuateIndex = 0;
      },

      ClassMethod(path) {
        if (features.includes('decorator')) {
          const {
            node
          } = path;

          if (node.decorators && node.decorators.length > 0) {
            const enode = evacuateClassMethodDecoratorsNode(node, {
              evacuatedTagName,
              index: evacuateIndex++
            });
            node.decorators = [];
            path.insertBefore(enode);
          }
        }
      },

      ClassProperty(path) {
        if (features.includes('classProperties')) {
          const {
            node
          } = path;
          const enode = evacuateClassPropNode(node, {
            evacuatedTagName,
            index: evacuateIndex++
          });
          path.replaceWith(enode);
        }
      }

    }
  };
});
var _default = evacuateFeaturesPlugin;
exports.default = _default;
//# sourceMappingURL=evacuate.js.map