"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var t = _interopRequireWildcard(require("@babel/types"));

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _util = require("./util");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 *
 */
function restoreEvacuatedCode(path, commentType, evacuatedTagRegex, container) {
  let remainingComments = [];
  const evacuatedNodes = [];
  const commentsProp = `${commentType}Comments`;
  const comments = path.node[commentsProp] || [];

  for (const comment of comments) {
    const {
      value: commentText
    } = comment;

    if (evacuatedTagRegex.test(commentText)) {
      const json = commentText.replace(evacuatedTagRegex, '');

      try {
        const node = JSON.parse(json);
        node.leadingComments = [...remainingComments, ...(node.leadingComments || [])];
        remainingComments = [];
        evacuatedNodes.push(node);
      } catch (e) {
        console.error(e);
      }
    } else {
      remainingComments.push((0, _util.stripLocationFromNode)(comment));
    }
  }

  if (evacuatedNodes.length > 0) {
    path.node[commentsProp] = remainingComments;
    const evacuatedNodes_ = commentType === 'leading' ? evacuatedNodes : evacuatedNodes.reverse();

    for (const evacuatedNode of evacuatedNodes_) {
      if (container) {
        path.pushContainer(container, evacuatedNode);
      } else {
        if (commentType === 'leading') {
          path.insertBefore(evacuatedNode);
        } else {
          path.insertAfter(evacuatedNode);
        }
      }
    }
  }
}

function parseEvacuatedComment(comment, {
  evacuatedTagName
}) {
  const regexp = new RegExp(`^[\\s\\*]*@preserve\\s[\\s\\*]*@${evacuatedTagName}\\s[\\s\\*]*(\\w+)\\s+(\\w+)(=|\\(\\))?`);
  const m = comment.match(regexp);

  if (m) {
    const [_, type, identifier, mod] = m;
    return {
      type,
      identifier,
      mod
    };
  }

  return null;
}

function createClassPropNode(identifier, mod, arg) {
  return mod === '=' ? t.classProperty(t.identifier(identifier), arg) : t.classProperty(t.identifier(identifier));
}

function createDecoratorNode(identifier, mod, args) {
  return t.decorator(mod === '()' ? t.callExpression(t.identifier(identifier), args.elements) : t.identifier(identifier));
}

function unevacuateDecoratorNode(decorator) {
  if (decorator.type !== 'ArrayExpression') {
    return;
  }

  const [name, ...args] = decorator.elements;

  if (!name || name.type !== 'StringLiteral') {
    return;
  }

  return t.decorator(args.length > 0 ? t.callExpression(t.identifier(name.value), (0, _util.stripLocationFromNode)(args)) : t.identifier(name.value));
}

function unevacuateCallNode(expression) {
  const [type, name, decorators, value] = expression.arguments;

  if (!type || type.type !== 'StringLiteral') {
    return;
  }

  if (!name || name.type !== 'StringLiteral') {
    return;
  }

  if (!decorators || decorators.type !== 'ArrayExpression') {
    return;
  }

  const unode = type.value === 'classProperty' ? t.classProperty(t.identifier(name.value), ...(value ? [(0, _util.stripLocationFromNode)(value)] : [])) : type.value === 'classMethod' ? t.classMethod('method', t.identifier(name.value), [], t.blockStatement([])) : undefined;

  if (!unode) {
    return;
  }

  unode.decorators = decorators.elements.map(unevacuateDecoratorNode).filter(Boolean);
  return unode;
}
/**
 *
 */


function applyDecoratorsToClassMethod(path, methodNode) {
  const classBodyPath = path.parentPath;
  const body = classBodyPath.node.body;

  for (let i = 0; i < body.length; i++) {
    const entryPath = classBodyPath.get(`body.${i}`);
    const entry = entryPath.node;

    if (entry.type === 'ClassMethod' && entry.key.name === methodNode.key.name) {
      entry.decorators = (0, _util.stripLocationFromNode)(methodNode.decorators);
      break;
    }
  }

  path.replaceWith(t.noop());
}
/**
 *
 */


const unevacuateFeaturesPlugin = (0, _helperPluginUtils.declare)((api, options) => {
  api.assertVersion(7);
  const {
    evacuatedTagName = 'evacuated'
  } = options;
  const evacuatedMethodRegex = new RegExp(`^__${evacuatedTagName}_`);
  return {
    name: 'transform-unevacuate-features',
    visitor: {
      ClassMethod(path) {
        const {
          node
        } = path;

        if (node.key.type === 'Identifier' && evacuatedMethodRegex.test(node.key.name)) {
          const stmt = node.body.body[0];

          if (stmt.type !== 'ExpressionStatement') {
            return;
          }

          const {
            expression
          } = stmt;

          if (expression.type !== 'CallExpression') {
            return;
          }

          const unode = unevacuateCallNode(expression);

          if (unode) {
            if (unode.type === 'ClassProperty') {
              path.replaceWith(unode);
            } else if (unode.type === 'ClassMethod') {
              applyDecoratorsToClassMethod(path, unode);
            }
          }
        }
      }

    }
  };
});
var _default = unevacuateFeaturesPlugin;
exports.default = _default;
//# sourceMappingURL=unevacuate.js.map